@@ -7,6 +7,8 @@ Python's :py:class:`argparse.ArgumentParser` and a ``configs`` dictionary, which
 This module extends the parser object :py:class:`sfftkrw.core.parser.Parser` as well as includes a
 :py:func:`sfftk.core.parser.parse_args` function which does sanity checking of all command line arguments.
 """
+import argparse
+import configparser
 import os
 import re
 import sys
@@ -17,6 +19,7 @@ from sfftkrw.core import _dict_iter_keys, _decode, _input, _str, _xrange
 from sfftkrw.core.parser import Parser, subparsers, convert_parser, view_parser, tests_parser, tool_list, add_args
 from sfftkrw.core.print_tools import print_date
 
+from ..core.prep import check_mask_is_binary
 from ..notes import RESOURCE_LIST
 
 __author__ = 'Paul K. Korir, PhD'
@@ -332,6 +335,50 @@ transform_prep_parser.add_argument(
     help="infix to be added to filenames e.g. file.stl -> file_<infix>.stl [default: 'transformed']",
 )
 add_args(transform_prep_parser, verbose)
+# =========================================================================
+# prep: mergemask
+# =========================================================================
+mergemask_prep_parser = prep_subparsers.add_parser(
+    'mergemask',
+    description='Merge two or more binary masks into one with integer labels',
+    help='merge two or more binary masks',
+)
+add_args(mergemask_prep_parser, config_path)
+add_args(mergemask_prep_parser, shipped_configs)
+add_args(mergemask_prep_parser, verbose)
+mergemask_prep_parser.add_argument(
+    'masks',
+    nargs='+',
+    help="a sequence of two masks in a CCP4-like format e.g. .mrc, .map, .rec; "
+         "the order of placement determines the order of labels"
+)
+mergemask_prep_parser.add_argument(
+    '-P', '--output-prefix',
+    default="merged_mask",
+    help="the prefix to use for the output; two files are written to the output: merged_mask.mrc and merged_mask.json "
+         "(metadata showing the mask-label relations)"
+)
+mergemask_prep_parser.add_argument(
+    '--mask-extension',
+    default="mrc",
+    help="the file extension to use [default: 'mrc']"
+)
+mergemask_prep_parser.add_argument(
+    '--overwrite',
+    action='store_true',
+    help="if the output already exists overwrite it [default: False]"
+)
+mergemask_prep_parser.add_argument(
+    '--skip-assessment',
+    action='store_true',
+    help="use this option to avoid the lengthy task of assessing the masks to make sure they are binary [default: False]"
+)
+mergemask_prep_parser.add_argument(
+    '--allow-overlap',
+    action='store_true',
+    help="allow masks to have overlapping voxels; this results in a complex hierarchy of labels described in the "
+         "label_tree [default: False]"
+)
 
 # =========================================================================
 # convert subparser
@@ -348,7 +395,9 @@ convert_parser.add_argument(
     help="for segments structured hierarchically (e.g. Segger from UCSF Chimera and Chimera X) "
          "convert all segment leves in the hierarchy [default: False]"
 )
-convert_parser.add_argument(
+multi_or_label_mutex_parser = convert_parser.add_mutually_exclusive_group(required=False)
+# convert_parser.add_argument(
+multi_or_label_mutex_parser.add_argument(
     '-m', '--multi-file',
     action='store_true',
     default=False,
@@ -364,6 +413,12 @@ convert_parser.add_argument(
     '--image',
     help="specify the segmented EMDB MAP/MRC file from which to determine the correct image-to-physical transform"
 )
+multi_or_label_mutex_parser.add_argument(
+    '--label-tree',
+    help="a JSON file produced by running 'sff prep mergemask' which captures: "
+         "1) the mask labels (key: 'mask_to_label') and "
+         "2) the hierarchical relationship between labels (key: 'label_tree')"
+)
 
 # =========================================================================
 # config subparser
@@ -1009,6 +1064,74 @@ def _set_subtype_index(args, ext):
     return args
 
 
+def _masks_exist(args: argparse.Namespace) -> bool:
+    """Test whether all mask files exist"""
+    all_exist = True
+    for mask in args.masks:
+        if not os.path.exists(mask):
+            all_exist = all_exist and False
+            if args.verbose:
+                print_date(f"missing mask: {mask}")
+    return all_exist
+
+
+def _mask_all_correct_files(args: argparse.Namespace) -> bool:
+    """Test that all masks have the right file format"""
+    all_masks = True
+    for mask in args.masks:
+        if not re.search(r"(map|mrc|rec)$", mask, re.IGNORECASE):
+            all_masks = all_masks and False
+            if args.verbose:
+                print_date(f"invalid extension {mask}")
+    return all_masks
+
+
+def _masks_have_same_dimensions(args: argparse.Namespace) -> bool:
+    """Test that a list of paths to masks have the same dimensions"""
+    dimensions = list()
+    from ..readers.mapreader import Map
+    if hasattr(args, 'masks'):
+        masks = args.masks
+    elif hasattr(args, 'from_file'):
+        masks = args.from_file
+    else:
+        raise ValueError('args missing attribute with masks')
+    for filename in masks:
+        this_map = Map(filename, header_only=True)
+        dimension = (this_map._nc, this_map._nr, this_map._ns)
+        if args.verbose:
+            print_date(f"info: mask {filename} has dimension {dimension} ")
+        dimensions.append(dimension)
+    dimension_comparisons = list(map(lambda d: d == dimensions[0], dimensions))
+    return all(dimension_comparisons)
+
+
+# def _masks_have_same_mode(args: argparse.Namespace) -> bool:
+#     """Test that a list of paths to masks have the same mode"""
+#     modes = list()
+#     from ..readers.mapreader import Map
+#     for filename in args.masks:
+#         this_map = Map(filename, header_only=True)
+#         mode = this_map._mode
+#         if args.verbose:
+#             print_date(f"info: mask {filename} has mode {mode} ")
+#         modes.append(mode)
+#     modes_comparisons = list(map(lambda d: d == modes[0], modes))
+#     return all(modes_comparisons)
+
+
+def _masks_have_mode_zero(args: argparse.Namespace) -> bool:
+    """Test that mode must be zero to proceed"""
+    from ..readers.mapreader import Map
+    for filename in args.masks:
+        this_map = Map(filename, header_only=True)
+        if this_map._mode != 0:
+            if args.verbose:
+                print_date(f"error: mask {filename} has mode {this_map._mode}; mode must be 0")
+            return False
+    return True
+
+
 # parser function
 def parse_args(_args, use_shlex=False):
     """
@@ -1150,6 +1273,7 @@ def parse_args(_args, use_shlex=False):
                     return 64, configs
                 if args.verbose:
                     print_date("Output will be written to {}".format(args.output))
+        # transform
         elif args.prep_subcommand == 'transform':
             ext = args.from_file.split('.')[-1]
             if ext.lower() not in RESCALABLE_FILE_FORMATS:
@@ -1163,6 +1287,27 @@ def parse_args(_args, use_shlex=False):
                     return 64, configs
                 if args.verbose:
                     print_date("Output will be written to {}".format(args.output))
+        # mergemask
+        elif args.prep_subcommand == 'mergemask':
+            if len(args.masks) < 2:
+                print_date("error: mergemask requires two or more masks")
+                return 64, configs
+            if len(args.masks) > 255:
+                print_date(f"error: mergemask can handle at most 255 masks ({len(args.masks)} provided)")
+                return 64, configs
+            if not _masks_exist(args):
+                print_date(f"error: one or more masks missing; please verify that all paths are correct")
+                return 65, configs
+            if not _mask_all_correct_files(args):
+                print_date(f"error: one or more invalid file formats; please retry")
+                return 65, configs
+            if not _masks_have_same_dimensions(args):
+                print_date(
+                    f"error: inhomogenious masks: dimension differs between masks (use --verbose to view details)")
+                return 65, configs
+            if not _masks_have_mode_zero(args):
+                print_date(f"error: mode must be zero (0); please run `sff prep binmap` first on all masks")
+                return 65, configs
 
     # view
     elif args.subcommand == 'view':
@@ -1184,8 +1329,7 @@ def parse_args(_args, use_shlex=False):
                 print_date("Invalid file type for --image. Please use .map, .mrc or .rec files only.")
                 return 65, configs  # 65 = DATAERR
         else:
-            print("Warning: missing --image <file.map> option to accurately determine image-to-physical transform",
-                  file=sys.stderr)
+            print_date("Warning: missing --image <file.map> option to accurately determine image-to-physical transform", stream=sys.stderr)
         # convert details to unicode
         if args.details is not None:
             args.details = _decode(args.details, 'utf-8')
@@ -1203,6 +1347,14 @@ def parse_args(_args, use_shlex=False):
             # check if this is an ambiguous extension
             if ext in EXTENSION_SUBTYPE_INDICES.keys():
                 args = _set_subtype_index(args, ext)
+            # now, let's check that this file is strictly binary
+            if re.match(r".*\.(map|mrc|rec)$", args.from_file, re.IGNORECASE):
+                if not check_mask_is_binary(args.from_file, verbose=args.verbose) and args.label_tree is None:
+                    print_date(
+                        "Error: non-binary mask; either use a binary mask or include the label "
+                        "tree with --label-tree flag"
+                    )
+                    return 65, configs
         else:
             if args.multi_file:
                 is_valid_format, file_format, invalid_formats = check_multi_file_formats(args.from_file)
@@ -1210,12 +1362,17 @@ def parse_args(_args, use_shlex=False):
                     file_missing = False
                     for fn in args.from_file:
                         try:
-                            assert os.path.exists(fn)
+                            assert os.path.exists(fn)  # check that all files exist
                         except AssertionError:
                             print_date("File {} was not found".format(fn))
                             file_missing = True
                     if file_missing:
                         return 64, configs
+                    # none of the files are missing and the extensions are OK
+                    # now check for homogeneity of CCP4/MRC masks
+                    if re.search(r"(map|mrc|rec)$", file_format, re.IGNORECASE):
+                        if not _masks_have_same_dimensions(args):
+                            return 65, configs
                 else:
                     print_date("Invalid format(s) for multi-file segmentation: {}; should be only one of: {}".format(
                         ', '.join(invalid_formats),
@@ -1548,3 +1705,10 @@ external reference IDs for segment {}".format(args.segment_id))
                 args.output = args.other
 
     return args, configs
+
+
+def cli(cmd: str) -> (argparse.Namespace, configparser.ConfigParser):
+    """CLI function"""
+    import shlex
+    sys.argv = shlex.split(cmd)
+    return parse_args(sys.argv)
